@@ -44,6 +44,10 @@
 #define NO_BLT_FILL 0
 #define NO_BLT_FILL_BOXES 0
 
+#ifndef PICT_TYPE_BGRA
+#define PICT_TYPE_BGRA 8
+#endif
+
 static const uint8_t copy_ROP[] = {
 	ROP_0,                  /* GXclear */
 	ROP_DSa,                /* GXand */
@@ -1721,6 +1725,7 @@ blt_put_composite(struct sna *sna,
 				     data, pitch, src_x, src_y,
 				     &box, 1);
 		assert(ok);
+		(void)ok;
 	}
 }
 
@@ -1757,6 +1762,7 @@ fastcall static void blt_put_composite_box(struct sna *sna,
 				     op->u.blt.sx, op->u.blt.sy,
 				     box, 1);
 		assert(ok);
+		(void)ok;
 	}
 }
 
@@ -1794,6 +1800,7 @@ static void blt_put_composite_boxes(struct sna *sna,
 				     op->u.blt.sx, op->u.blt.sy,
 				     box, n);
 		assert(ok);
+		(void)ok;
 	}
 }
 
@@ -2243,7 +2250,7 @@ fill:
 	tmp->dst.bo = sna_drawable_use_bo(dst->pDrawable, hint,
 					  &dst_box, &tmp->damage);
 
-	if (hint & REPLACES)
+	if (tmp->dst.bo && hint & REPLACES)
 		kgem_bo_undo(&sna->kgem, tmp->dst.bo);
 
 	ret = false;
@@ -2997,7 +3004,7 @@ bool sna_blt_copy_boxes(struct sna *sna, uint8_t alu,
 
 	if (kgem->nexec > 1 && __kgem_ring_empty(kgem)) {
 		_kgem_submit(kgem);
-	} else if (kgem->gen >= 060 && kgem_check_batch(kgem, 3)) {
+	} else if (kgem->gen >= 060 && src_bo == dst_bo && kgem_check_batch(kgem, 3)) {
 		uint32_t *b = kgem->batch + kgem->nbatch;
 		b[0] = XY_SETUP_CLIP;
 		b[1] = b[2] = 0;
