@@ -62,7 +62,7 @@ static inline bool region_is_singular(RegionRec *r)
 
 #if HAS_DEBUG_FULL
 static const char *_debug_describe_region(char *buf, int max,
-					  RegionPtr region)
+					  const RegionRec *region)
 {
 	BoxPtr extents;
 	BoxPtr box;
@@ -116,7 +116,7 @@ static const char *_debug_describe_region(char *buf, int max,
 }
 
 static const char *_debug_describe_damage(char *buf, int max,
-					  struct sna_damage *damage)
+					  const struct sna_damage *damage)
 {
 	char damage_str[500], region_str[500];
 	int str_max;
@@ -415,7 +415,7 @@ static void __sna_damage_reduce(struct sna_damage *damage)
 	assert(damage->mode != DAMAGE_ALL);
 	assert(damage->dirty);
 
-	DBG(("    reduce: before region.n=%ld\n", REGION_NUM_RECTS(region)));
+	DBG(("    reduce: before region.n=%ld\n", (long)REGION_NUM_RECTS(region)));
 
 	nboxes = damage->embedded_box.size;
 	list_for_each_entry(iter, &damage->embedded_box.list, list)
@@ -529,7 +529,7 @@ done:
 	free_list(&damage->embedded_box.list);
 	reset_embedded_box(damage);
 
-	DBG(("    reduce: after region.n=%ld\n", REGION_NUM_RECTS(region)));
+	DBG(("    reduce: after region.n=%ld\n", (long)REGION_NUM_RECTS(region)));
 }
 
 static void damage_union(struct sna_damage *damage, const BoxRec *box)
@@ -1332,18 +1332,57 @@ int _sna_damage_contains_box(struct sna_damage *damage,
 }
 #endif
 
+static bool box_overlaps(const BoxRec *a, const BoxRec *b)
+{
+	return (a->x1 < b->x2 && a->x2 > b->x1 &&
+		a->y1 < b->y2 && a->y2 > b->y1);
+}
+
 bool _sna_damage_contains_box__no_reduce(const struct sna_damage *damage,
 					 const BoxRec *box)
 {
+	int n, count;
+	const BoxRec *b;
+
 	assert(damage && damage->mode != DAMAGE_ALL);
-	if (damage->mode == DAMAGE_SUBTRACT)
+	if (!box_contains(&damage->extents, box))
 		return false;
 
-	if (!sna_damage_overlaps_box(damage, box))
-		return false;
+	n = pixman_region_contains_rectangle((pixman_region16_t *)&damage->region, (BoxPtr)box);
+	if (!damage->dirty)
+		return n == PIXMAN_REGION_IN;
 
-	return pixman_region_contains_rectangle((RegionPtr)&damage->region,
-						(BoxPtr)box) == PIXMAN_REGION_IN;
+	if (damage->mode == DAMAGE_ADD) {
+		if (n == PIXMAN_REGION_IN)
+			return true;
+
+		count = damage->embedded_box.size;
+		if (list_is_empty(&damage->embedded_box.list))
+			count -= damage->remain;
+
+		b = damage->embedded_box.box;
+		for (n = 0; n < count; n++) {
+			if (box_contains(&b[n], box))
+				return true;
+		}
+
+		return false;
+	} else {
+		if (n != PIXMAN_REGION_IN)
+			return false;
+
+		if (!list_is_empty(&damage->embedded_box.list))
+			return false;
+
+		count = damage->embedded_box.size - damage->remain;
+		b = damage->embedded_box.box;
+		for (n = 0; n < count; n++) {
+			if (box_overlaps(&b[n], box))
+				return false;
+		}
+
+		return true;
+	}
 }
 
 static bool __sna_damage_intersect(struct sna_damage *damage,
