@@ -151,7 +151,7 @@ struct sna_composite_op {
 struct sna_opacity_box {
 	BoxRec box;
 	float alpha;
-} __packed__;
+} tightly_packed;
 
 struct sna_composite_spans_op {
 	struct sna_composite_op base;
@@ -287,7 +287,7 @@ struct sna_render {
 	struct sna_solid_cache {
 		struct kgem_bo *cache_bo;
 		struct kgem_bo *bo[1024];
-		uint32_t color[1025];
+		uint32_t color[1024];
 		int last;
 		int size;
 		int dirty;
